@@ -17,11 +17,22 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <gtk/gtk.h>
+#include <stdlib.h>
+
+#include <audacious/gtk-compat.h>
+#include <audacious/misc.h>
+#include <audacious/playlist.h>
+#include <libaudcore/audstrings.h>
+#include <libaudgui/libaudgui.h>
+#include <libaudgui/libaudgui-gtk.h>
+
 #include "filewriter.h"
 #include "plugins.h"
+#include "convert.h"
 
 struct format_info input;
 
@@ -46,8 +57,8 @@ enum fileext_t
     FILEEXT_MAX
 };
 
-static gint fileext = WAV;
-static gchar *fileext_str[] =
+static gint fileext;
+static const gchar *fileext_str[FILEEXT_MAX] =
 {
     "wav",
 #ifdef FILEWRITER_MP3
@@ -61,326 +72,298 @@ static gchar *fileext_str[] =
 #endif
 };
 
-static FileWriter plugin;
+static FileWriter *plugin;
 
 static GtkWidget *saveplace_hbox, *saveplace;
-static gboolean save_original = TRUE;
+static gboolean save_original;
 
 static GtkWidget *filenamefrom_hbox, *filenamefrom_label, *filenamefrom_toggle;
-static gboolean filenamefromtags = TRUE;
+static gboolean filenamefromtags;
 
 static GtkWidget *use_suffix_toggle = NULL;
-static gboolean use_suffix = FALSE;
+static gboolean use_suffix;
 
 static GtkWidget *prependnumber_toggle;
-static gboolean prependnumber = FALSE;
+static gboolean prependnumber;
 
-static gchar *file_path = NULL;
+static gchar *file_path;
 
-static void file_init(void);
-static void file_about(void);
-static gint file_open(AFormat fmt, gint rate, gint nch);
-static void file_write(void *ptr, gint length);
-static void file_close(void);
-static void file_flush(gint time);
-static void file_pause(short p);
-static gint file_free(void);
-static gint file_playing(void);
-static gint file_get_written_time(void);
-static gint file_get_output_time(void);
-static void file_configure(void);
+VFSFile *output_file = NULL;
+Tuple *tuple = NULL;
 
-OutputPlugin file_op =
-{
-    .description = "FileWriter Plugin",
-    .init = file_init,
-    .about = file_about,
-    .configure = file_configure,
-    .open_audio = file_open,
-    .write_audio = file_write,
-    .close_audio = file_close,
-    .flush = file_flush,
-    .pause = file_pause,
-    .buffer_free = file_free,
-    .buffer_playing = file_playing,
-    .output_time = file_get_output_time,
-    .written_time = file_get_written_time
+static gint64 samples_written;
+
+FileWriter *plugins[FILEEXT_MAX] = {
+    &wav_plugin,
+#ifdef FILEWRITER_MP3
+    &mp3_plugin,
+#endif
+#ifdef FILEWRITER_VORBIS
+    &vorbis_plugin,
+#endif
+#ifdef FILEWRITER_FLAC
+    &flac_plugin,
+#endif
 };
 
-OutputPlugin *file_oplist[] = { &file_op, NULL };
-
-SIMPLE_OUTPUT_PLUGIN(filewriter, file_oplist);
-
 static void set_plugin(void)
 {
     if (fileext < 0 || fileext >= FILEEXT_MAX)
         fileext = 0;
 
-    if (fileext == WAV)
-        plugin = wav_plugin;
-#ifdef FILEWRITER_MP3
-    if (fileext == MP3)
-        plugin = mp3_plugin;
-#endif
-#ifdef FILEWRITER_VORBIS
-    if (fileext == VORBIS)
-        plugin = vorbis_plugin;
-#endif
-#ifdef FILEWRITER_FLAC
-    if (fileext == FLAC)
-        plugin = flac_plugin;
-#endif
+    plugin = plugins[fileext];
 }
 
-static void file_init(void)
+static gint file_write_output (void * data, gint length)
 {
-    mcs_handle_t *db;
-    /*GtkWidget *menu_root;
+    return vfs_fwrite (data, 1, length, output_file);
+}
 
-    menu_root = gtk_menu_item_new_with_label(_("FileWriter"));
-    gtk_widget_show(menu_root);
-    audacious_menu_plugin_item_add(AUDACIOUS_MENU_PLAYLIST_RCLICK, menu_root);*/
+static const gchar * const filewriter_defaults[] = {
+ "fileext", "0", /* WAV */
+ "filenamefromtags", "TRUE",
+ "prependnumber", "FALSE",
+ "save_original", "TRUE",
+ "use_suffix", "FALSE",
+ NULL};
 
-    db = aud_cfg_db_open();
-    aud_cfg_db_get_int(db, "filewriter", "fileext", &fileext);
-    aud_cfg_db_get_string(db, "filewriter", "file_path", &file_path);
-    aud_cfg_db_get_bool(db, "filewriter", "save_original", &save_original);
-    aud_cfg_db_get_bool(db, "filewriter", "use_suffix", &use_suffix);
-    aud_cfg_db_get_bool(db, "filewriter", "filenamefromtags", &filenamefromtags);
-    aud_cfg_db_get_bool(db, "filewriter", "prependnumber", &prependnumber);
-    aud_cfg_db_close(db);
+static gboolean file_init (void)
+{
+    aud_config_set_defaults ("filewriter", filewriter_defaults);
 
-    if (!file_path)
-        file_path = g_strdup(g_get_home_dir());
+    fileext = aud_get_int ("filewriter", "fileext");
+    filenamefromtags = aud_get_bool ("filewriter", "filenamefromtags");
+    file_path = aud_get_string ("filewriter", "file_path");
+    prependnumber = aud_get_bool ("filewriter", "prependnumber");
+    save_original = aud_get_bool ("filewriter", "save_original");
+    use_suffix = aud_get_bool ("filewriter", "use_suffix");
+
+    if (! file_path[0])
+    {
+        g_return_val_if_fail (getenv ("HOME") != NULL, FALSE);
+        file_path = g_filename_to_uri (getenv ("HOME"), NULL, NULL);
+        g_return_val_if_fail (file_path != NULL, FALSE);
+    }
 
     set_plugin();
-    if (plugin.init)
-        plugin.init();
+    if (plugin->init)
+        plugin->init(&file_write_output);
+
+    return TRUE;
 }
 
-void file_about(void)
+static void file_cleanup (void)
 {
-    static GtkWidget *dialog;
-
-    if (dialog != NULL)
-        return;
-
-    dialog = audacious_info_dialog(_("About FileWriter-Plugin"),
-                               _("FileWriter-Plugin\n\n"
-                               "This program is free software; you can redistribute it and/or modify\n"
-                               "it under the terms of the GNU General Public License as published by\n"
-                               "the Free Software Foundation; either version 2 of the License, or\n"
-                               "(at your option) any later version.\n"
-                               "\n"
-                               "This program is distributed in the hope that it will be useful,\n"
-                               "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-                               "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-                               "GNU General Public License for more details.\n"
-                               "\n"
-                               "You should have received a copy of the GNU General Public License\n"
-                               "along with this program; if not, write to the Free Software\n"
-                               "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,\n"
-                               "USA."), _("Ok"), FALSE, NULL, NULL);
-    gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-                       GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
+    g_free (file_path);
+    file_path = NULL;
 }
 
-static gint file_open(AFormat fmt, gint rate, gint nch)
+void file_about (void)
+{
+    static GtkWidget * dialog;
+
+    audgui_simple_message (& dialog, GTK_MESSAGE_INFO,
+     _("About FileWriter-Plugin"),
+     "FileWriter-Plugin\n\n"
+     "This program is free software; you can redistribute it and/or modify\n"
+     "it under the terms of the GNU General Public License as published by\n"
+     "the Free Software Foundation; either version 2 of the License, or\n"
+     "(at your option) any later version.\n"
+     "\n"
+     "This program is distributed in the hope that it will be useful,\n"
+     "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+     "GNU General Public License for more details.\n"
+     "\n"
+     "You should have received a copy of the GNU General Public License\n"
+     "along with this program; if not, write to the Free Software\n"
+     "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
+     "USA.");
+}
+
+static VFSFile * safe_create (const gchar * filename)
+{
+    if (! vfs_file_test (filename, G_FILE_TEST_EXISTS))
+        return vfs_fopen (filename, "w");
+
+    const gchar * extension = strrchr (filename, '.');
+    gint length = strlen (filename);
+    gchar scratch[length + 3];
+    gint count;
+
+    for (count = 1; count < 100; count ++)
+    {
+        if (extension == NULL)
+            sprintf (scratch, "%s-%d", filename, count);
+        else
+            sprintf (scratch, "%.*s-%d%s", (gint) (extension - filename),
+             filename, count, extension);
+
+        if (! vfs_file_test (scratch, G_FILE_TEST_EXISTS))
+            return vfs_fopen (scratch, "w");
+    }
+
+    return NULL;
+}
+
+static gint file_open(gint fmt, gint rate, gint nch)
 {
     gchar *filename = NULL, *temp = NULL;
-    const gchar *directory;
+    gchar * directory;
     gint pos;
     gint rv;
-    Playlist *playlist;
+    gint playlist;
 
     input.format = fmt;
     input.frequency = rate;
     input.channels = nch;
 
-    playlist = aud_playlist_get_active();
-    if(!playlist)
+    playlist = aud_playlist_get_playing ();
+    if (playlist < 0)
         return 0;
 
     pos = aud_playlist_get_position(playlist);
-    tuple = aud_playlist_get_tuple(playlist, pos);
-    if(!tuple)
+    tuple = aud_playlist_entry_get_tuple (playlist, pos, FALSE);
+    if (tuple == NULL)
         return 0;
 
     if (filenamefromtags)
     {
-        gchar *utf8 = aud_tuple_formatter_make_title_string(tuple, aud_get_gentitle_format());
+        gchar * utf8 = aud_playlist_entry_get_title (playlist, pos, FALSE);
+        string_replace_char (utf8, '/', ' ');
 
-        g_strchomp(utf8); //chop trailing ^J --yaz
+        gchar buf[3 * strlen (utf8) + 1];
+        str_encode_percent (utf8, -1, buf);
+        str_unref (utf8);
 
-        filename = g_locale_from_utf8(utf8, -1, NULL, NULL, NULL);
-        g_free(utf8);
-        while (filename != NULL && (temp = strchr(filename, '/')) != NULL)
-            *temp = '-';
+        filename = g_strdup (buf);
     }
-    if (filename == NULL)
+    else
     {
-        filename = g_strdup(aud_tuple_get_string(tuple, FIELD_FILE_NAME, NULL));
+        temp = aud_playlist_entry_get_filename (playlist, pos);
+        gchar * original = strrchr (temp, '/');
+        g_return_val_if_fail (original != NULL, 0);
+        filename = g_strdup (original + 1);
+        str_unref (temp);
+
         if (!use_suffix)
             if ((temp = strrchr(filename, '.')) != NULL)
                 *temp = '\0';
     }
-    if (filename == NULL)
-        filename = g_strdup_printf("aud-%d", pos);
-
 
     if (prependnumber)
     {
-        gint number = aud_tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL);
+        gint number = tuple_get_int(tuple, FIELD_TRACK_NUMBER, NULL);
         if (!tuple || !number)
             number = pos + 1;
 
-        temp = g_strdup_printf("%.02d %s", number, filename);
+        temp = g_strdup_printf ("%d%%20%s", number, filename);
         g_free(filename);
         filename = temp;
     }
 
     if (save_original)
-        directory = aud_tuple_get_string(tuple, FIELD_FILE_PATH, NULL);
+    {
+        temp = aud_playlist_entry_get_filename (playlist, pos);
+        directory = g_strdup (temp);
+        str_unref (temp);
+        temp = strrchr (directory, '/');
+        g_return_val_if_fail (temp != NULL, 0);
+        temp[1] = 0;
+    }
     else
-        directory = file_path;
+    {
+        g_return_val_if_fail (file_path[0], 0);
+        if (file_path[strlen (file_path) - 1] == '/')
+            directory = g_strdup (file_path);
+        else
+            directory = g_strdup_printf ("%s/", file_path);
+    }
 
-    temp = g_strdup_printf("file://%s/%s.%s",
-                           directory, filename, fileext_str[fileext]);
-    g_free(filename);
+    temp = g_strdup_printf ("%s%s.%s", directory, filename, fileext_str[fileext]);
+    g_free (directory);
+    g_free (filename);
     filename = temp;
 
-    output_file = aud_vfs_fopen(filename, "w");
-    g_free(filename);
+    output_file = safe_create (filename);
+    g_free (filename);
 
-    if (!output_file)
+    if (output_file == NULL)
         return 0;
 
-    rv = (plugin.open)();
+    convert_init(fmt, plugin->format_required, nch);
+
+    rv = (plugin->open)();
+
+    samples_written = 0;
 
     return rv;
 }
 
-static void convert_buffer(gpointer buffer, gint length)
-{
-    gint i;
-
-    if (input.format == FMT_S8)
-    {
-        guint8 *ptr1 = buffer;
-        gint8 *ptr2 = buffer;
-
-        for (i = 0; i < length; i++)
-            *(ptr1++) = *(ptr2++) ^ 128;
-    }
-    if (input.format == FMT_S16_BE)
-    {
-        gint16 *ptr = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr++)
-            *ptr = GUINT16_SWAP_LE_BE(*ptr);
-    }
-    if (input.format == FMT_S16_NE)
-    {
-        gint16 *ptr = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr++)
-            *ptr = GINT16_TO_LE(*ptr);
-    }
-    if (input.format == FMT_U16_BE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE(GUINT16_FROM_BE(*ptr2) ^ 32768);
-    }
-    if (input.format == FMT_U16_LE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE(GUINT16_FROM_LE(*ptr2) ^ 32768);
-    }
-    if (input.format == FMT_U16_NE)
-    {
-        gint16 *ptr1 = buffer;
-        guint16 *ptr2 = buffer;
-
-        for (i = 0; i < length >> 1; i++, ptr2++)
-            *(ptr1++) = GINT16_TO_LE((*ptr2) ^ 32768);
-    }
-}
-
 static void file_write(void *ptr, gint length)
 {
-    if (input.format == FMT_S8 || input.format == FMT_S16_BE ||
-        input.format == FMT_U16_LE || input.format == FMT_U16_BE ||
-        input.format == FMT_U16_NE)
-        convert_buffer(ptr, length);
-#ifdef WORDS_BIGENDIAN
-    if (input.format == FMT_S16_NE)
-        convert_buffer(ptr, length);
-#endif
+    int len = convert_process (ptr, length);
 
-    plugin.write(ptr, length);
+    plugin->write(convert_output, len);
+
+    samples_written += length / FMT_SIZEOF (input.format);
+}
+
+static void file_drain (void)
+{
 }
 
 static void file_close(void)
 {
-    plugin.close();
+    plugin->close();
+    convert_free();
 
-    if (output_file)
-    {
-        written = 0;
-        aud_vfs_fclose(output_file);
-    }
+    if (output_file != NULL)
+        vfs_fclose(output_file);
     output_file = NULL;
+
+    if (tuple)
+    {
+        tuple_unref (tuple);
+        tuple = NULL;
+    }
 }
 
 static void file_flush(gint time)
 {
-    if (time < 0)
-        return;
-
-    file_close();
-    file_open(input.format, input.frequency, input.channels);
-
-    offset = time;
+    samples_written = time * (gint64) input.channels * input.frequency / 1000;
 }
 
-static void file_pause(short p)
+static void file_set_written_time (gint time)
+{
+    /* Guesswork:
+     * If time = 0, we are starting a new song and need to open a new file.  If
+     * time > 0, we have already done this, and we are just setting the time
+     * counter. */
+    if (! time)
+    {
+        file_close ();
+        file_open (input.format, input.frequency, input.channels);
+    }
+
+    samples_written = time * (gint64) input.channels * input.frequency / 1000;
+}
+
+static void file_pause (gboolean p)
 {
 }
 
-static gint file_free(void)
+static gint file_get_time (void)
 {
-    return plugin.free();
-}
-
-static gint file_playing(void)
-{
-    return plugin.playing();
-}
-
-static gint file_get_written_time(void)
-{
-    return plugin.get_written_time();
-}
-
-static gint file_get_output_time(void)
-{
-    return file_get_written_time();
+    return samples_written * 1000 / (input.channels * input.frequency);
 }
 
 static void configure_ok_cb(gpointer data)
 {
-    mcs_handle_t *db;
-
     fileext = gtk_combo_box_get_active(GTK_COMBO_BOX(fileext_combo));
 
-    g_free(file_path);
-    file_path = g_strdup(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(path_dirbrowser)));
+    g_free (file_path);
+    file_path = gtk_file_chooser_get_uri ((GtkFileChooser *) path_dirbrowser);
 
     use_suffix =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_suffix_toggle));
@@ -388,15 +371,12 @@ static void configure_ok_cb(gpointer data)
     prependnumber =
         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prependnumber_toggle));
 
-    db = aud_cfg_db_open();
-    aud_cfg_db_set_int(db, "filewriter", "fileext", fileext);
-    aud_cfg_db_set_string(db, "filewriter", "file_path", file_path);
-    aud_cfg_db_set_bool(db, "filewriter", "save_original", save_original);
-    aud_cfg_db_set_bool(db, "filewriter", "filenamefromtags", filenamefromtags);
-    aud_cfg_db_set_bool(db, "filewriter", "use_suffix", use_suffix);
-    aud_cfg_db_set_bool(db, "filewriter", "prependnumber", prependnumber);
-
-    aud_cfg_db_close(db);
+    aud_set_int ("filewriter", "fileext", fileext);
+    aud_set_bool ("filewriter", "filenamefromtags", filenamefromtags);
+    aud_set_string ("filewriter", "file_path", file_path);
+    aud_set_bool ("filewriter", "prependnumber", prependnumber);
+    aud_set_bool ("filewriter", "save_original", save_original);
+    aud_set_bool ("filewriter", "use_suffix", use_suffix);
 
     gtk_widget_destroy(configure_win);
     if (path_dirbrowser)
@@ -407,14 +387,16 @@ static void fileext_cb(GtkWidget *combo, gpointer data)
 {
     fileext = gtk_combo_box_get_active(GTK_COMBO_BOX(fileext_combo));
     set_plugin();
+    if (plugin->init)
+        plugin->init(&file_write_output);
 
-    gtk_widget_set_sensitive(plugin_button, plugin.configure != NULL);
+    gtk_widget_set_sensitive(plugin_button, plugin->configure != NULL);
 }
 
 static void plugin_configure_cb(GtkWidget *button, gpointer data)
 {
-    if (plugin.configure)
-        plugin.configure();
+    if (plugin->configure)
+        plugin->configure();
 }
 
 
@@ -465,17 +447,15 @@ static void configure_destroy(void)
 
 static void file_configure(void)
 {
-    GtkTooltips *use_suffix_tooltips;
-
     if (!configure_win)
     {
         configure_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+        gtk_window_set_type_hint(GTK_WINDOW(configure_win), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-        gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
-                           GTK_SIGNAL_FUNC(configure_destroy), NULL);
-        gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
-                           GTK_SIGNAL_FUNC(gtk_widget_destroyed),
-                           &configure_win);
+        g_signal_connect (configure_win, "destroy", (GCallback)
+         configure_destroy, NULL);
+        g_signal_connect (configure_win, "destroy", (GCallback)
+         gtk_widget_destroyed, & configure_win);
 
         gtk_window_set_title(GTK_WINDOW(configure_win),
                              _("File Writer Configuration"));
@@ -493,23 +473,23 @@ static void file_configure(void)
         fileext_label = gtk_label_new(_("Output file format:"));
         gtk_box_pack_start(GTK_BOX(fileext_hbox), fileext_label, FALSE, FALSE, 0);
 
-        fileext_combo = gtk_combo_box_new_text();
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "WAV");
+        fileext_combo = gtk_combo_box_text_new ();
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "WAV");
 #ifdef FILEWRITER_MP3
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "MP3");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "MP3");
 #endif
 #ifdef FILEWRITER_VORBIS
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "Vorbis");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "Vorbis");
 #endif
 #ifdef FILEWRITER_FLAC
-        gtk_combo_box_append_text(GTK_COMBO_BOX(fileext_combo), "FLAC");
+        gtk_combo_box_text_append_text ((GtkComboBoxText *) fileext_combo, "FLAC");
 #endif
         gtk_box_pack_start(GTK_BOX(fileext_hbox), fileext_combo, FALSE, FALSE, 0);
         gtk_combo_box_set_active(GTK_COMBO_BOX(fileext_combo), fileext);
         g_signal_connect(G_OBJECT(fileext_combo), "changed", G_CALLBACK(fileext_cb), NULL);
 
         plugin_button = gtk_button_new_with_label(_("Configure"));
-        gtk_widget_set_sensitive(plugin_button, plugin.configure != NULL);
+        gtk_widget_set_sensitive(plugin_button, plugin->configure != NULL);
         g_signal_connect(G_OBJECT(plugin_button), "clicked", G_CALLBACK(plugin_configure_cb), NULL);
         gtk_box_pack_end(GTK_BOX(fileext_hbox), plugin_button, FALSE, FALSE, 0);
 
@@ -529,13 +509,11 @@ static void file_configure(void)
 
         saveplace = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(saveplace),
                                                                 _("Save into custom directory"));
-        g_signal_connect(G_OBJECT(saveplace), "toggled", G_CALLBACK(saveplace_custom_cb), NULL);
-        gtk_box_pack_start(GTK_BOX(saveplace_hbox), saveplace, FALSE, FALSE, 0);
-
         if (!save_original)
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(saveplace), TRUE);
 
-
+        g_signal_connect(G_OBJECT(saveplace), "toggled", G_CALLBACK(saveplace_custom_cb), NULL);
+        gtk_box_pack_start(GTK_BOX(saveplace_hbox), saveplace, FALSE, FALSE, 0);
 
         path_hbox = gtk_hbox_new(FALSE, 5);
         gtk_box_pack_start(GTK_BOX(configure_vbox), path_hbox, FALSE, FALSE, 0);
@@ -546,8 +524,7 @@ static void file_configure(void)
         path_dirbrowser =
             gtk_file_chooser_button_new (_("Pick a folder"),
                                          GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(path_dirbrowser),
-                                            file_path);
+        gtk_file_chooser_set_uri ((GtkFileChooser *) path_dirbrowser, file_path);
         gtk_box_pack_start(GTK_BOX(path_hbox), path_dirbrowser, TRUE, TRUE, 0);
 
         if (save_original)
@@ -586,9 +563,6 @@ static void file_configure(void)
         use_suffix_toggle = gtk_check_button_new_with_label(_("Don't strip file name extension"));
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_suffix_toggle), use_suffix);
         gtk_box_pack_start(GTK_BOX(configure_vbox), use_suffix_toggle, FALSE, FALSE, 0);
-        use_suffix_tooltips = gtk_tooltips_new();
-        gtk_tooltips_set_tip(use_suffix_tooltips, use_suffix_toggle, _("If enabled, the extension from the original filename will not be stripped before adding the new file extension to the end."), NULL);
-        gtk_tooltips_enable(use_suffix_tooltips);
 
         if (filenamefromtags)
             gtk_widget_set_sensitive(use_suffix_toggle, FALSE);
@@ -610,20 +584,18 @@ static void file_configure(void)
         configure_bbox = gtk_hbutton_box_new();
         gtk_button_box_set_layout(GTK_BUTTON_BOX(configure_bbox),
                                   GTK_BUTTONBOX_END);
-        gtk_button_box_set_spacing(GTK_BUTTON_BOX(configure_bbox), 5);
         gtk_box_pack_start(GTK_BOX(configure_vbox), configure_bbox,
                            FALSE, FALSE, 0);
 
         configure_cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-        gtk_signal_connect_object(GTK_OBJECT(configure_cancel), "clicked",
-                                  GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                                  GTK_OBJECT(configure_win));
+        g_signal_connect_swapped (configure_cancel, "clicked", (GCallback)
+         gtk_widget_destroy, configure_win);
         gtk_box_pack_start(GTK_BOX(configure_bbox), configure_cancel,
                            TRUE, TRUE, 0);
 
         configure_ok = gtk_button_new_from_stock(GTK_STOCK_OK);
-        gtk_signal_connect(GTK_OBJECT(configure_ok), "clicked",
-                           GTK_SIGNAL_FUNC(configure_ok_cb), NULL);
+        g_signal_connect (configure_ok, "clicked", (GCallback) configure_ok_cb,
+         NULL);
         gtk_box_pack_start(GTK_BOX(configure_bbox), configure_ok,
                            TRUE, TRUE, 0);
 
@@ -631,7 +603,21 @@ static void file_configure(void)
     }
 }
 
-VFSFile *output_file = NULL;
-guint64 written = 0;
-guint64 offset = 0;
-Tuple *tuple = NULL;
+AUD_OUTPUT_PLUGIN
+(
+ .name = "FileWriter",
+ .init = file_init,
+ .cleanup = file_cleanup,
+ .about = file_about,
+ .configure = file_configure,
+ .probe_priority = 0,
+ .open_audio = file_open,
+ .close_audio = file_close,
+ .write_audio = file_write,
+ .drain = file_drain,
+ .written_time = file_get_time,
+ .output_time = file_get_time,
+ .pause = file_pause,
+ .flush = file_flush,
+ .set_written_time = file_set_written_time,
+)
