@@ -26,6 +26,7 @@
 #include <libaudgui/libaudgui-gtk.h>
 
 #include <glib.h>
+#include <string.h>
 
 #define MIN_BPM         1
 #define MAX_BPM         512
@@ -252,7 +253,7 @@ static Tuple *metronom_probe_for_tuple(const gchar * filename, VFSFile *fd)
     gchar *tmp = NULL;
 
     if (metronom_get_cp(filename, &metronom, &tmp))
-        tuple_associate_string(tuple, FIELD_TITLE, NULL, tmp);
+        tuple_set_str(tuple, FIELD_TITLE, NULL, tmp);
 
     g_free(tmp);
 
