@@ -23,8 +23,10 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
 #include <gtk/gtk.h>
 
